package
{
	import away3d.entities.Mesh;
	import away3d.extrusions.Elevation;
	import away3d.lights.DirectionalLight;
	import away3d.materials.TextureMaterial;
	import away3d.materials.lightpickers.StaticLightPicker;
	import away3d.materials.methods.EnvMapMethod;
	import away3d.materials.methods.FogMethod;
	import away3d.materials.methods.FresnelSpecularMethod;
	import away3d.materials.methods.SimpleWaterNormalMethod;
	import away3d.primitives.PlaneGeometry;
	import away3d.primitives.SkyBox;
	import away3d.textures.BitmapCubeTexture;
	import away3d.textures.BitmapTexture;
	import away3d.utils.Cast;

	import com.blitting.display.Away3dFirstPersonViewport;

	import flash.display.BitmapData;

	[SWF(percentWidth = 100, percentHeight = 100, backgroundColor = 0x0, frameRate = 60)]
	public class Away3dTerrainExample extends Away3dFirstPersonViewport
	{

		//------------------------------
		//  constant
		//------------------------------

		[Embed(source = "assets/terrain/terrain-heightmap.png")]
		protected static const HeightMap:Class;

		[Embed(source = "assets/water/water_normals.jpg")]
		protected static const WaterNormals:Class;

		[Embed(source = "assets/skybox/space/space_posX.jpg")]
		protected static const EnvPosX:Class;

		[Embed(source = "assets/skybox/space/space_posY.jpg")]
		protected static const EnvPosY:Class;

		[Embed(source = "assets/skybox/space/space_posZ.jpg")]
		protected static const EnvPosZ:Class;

		[Embed(source = "assets/skybox/space/space_negX.jpg")]
		protected static const EnvNegX:Class;

		[Embed(source = "assets/skybox/space/space_negY.jpg")]
		protected static const EnvNegY:Class;

		[Embed(source = "assets/skybox/space/space_negZ.jpg")]
		protected static const EnvNegZ:Class;


		//------------------------------
		//  model
		//------------------------------

		protected var cubeTexture:BitmapCubeTexture;

		private var terrain:Elevation;

		private var waterMethod:SimpleWaterNormalMethod;

		private var waterMaterial:TextureMaterial;

		private var fresnelMethod:FresnelSpecularMethod;

		protected var terrainMaterial:TextureMaterial;

		private var sunLight:DirectionalLight;

		private var lightPicker:StaticLightPicker;

		protected var fogMethod:FogMethod;

		private var plane:Mesh;


		//------------------------------
		//  lifecycle
		//------------------------------

		public function Away3dTerrainExample()
		{
			super();
		}

		override protected function initializeCamera():void
		{
			super.initializeCamera();

			camera.lens.far = 14000;
			camera.lens.near = .05;
		}

		override protected function initializeLights():void
		{
			super.initializeLights();

			sunLight = new DirectionalLight(-300, -300, -5000);
			sunLight.color = 0xfffdc5;
			sunLight.ambient = 1;
			scene.addChild(sunLight);

			lightPicker = new StaticLightPicker([ sunLight ]);

			// create a global fog method
			fogMethod = new FogMethod(0, 8000, 0xcfd9de);
		}

		override protected function initializeMaterials():void
		{
			super.initializeMaterials();

			cubeTexture = new BitmapCubeTexture(Cast.bitmapData(EnvPosX),
												Cast.bitmapData(EnvNegX),
												Cast.bitmapData(EnvPosY),
												Cast.bitmapData(EnvNegY),
												Cast.bitmapData(EnvPosZ),
												Cast.bitmapData(EnvNegZ));

			terrainMaterial = new TextureMaterial(Cast.bitmapTexture(HeightMap));
			//			terrainMaterial.diffuseMethod = terrainMethod;
			//			terrainMaterial.normalMap = Cast.bitmapTexture(Normals);
			terrainMaterial.lightPicker = lightPicker;
			terrainMaterial.ambientColor = 0x303040;
			terrainMaterial.ambient = 1;
			terrainMaterial.specular = .2;
			terrainMaterial.addMethod(fogMethod);

			waterMethod = new SimpleWaterNormalMethod(Cast.bitmapTexture(WaterNormals), Cast.bitmapTexture(WaterNormals));
			fresnelMethod = new FresnelSpecularMethod();
			fresnelMethod.normalReflectance = .3;

			waterMaterial = new TextureMaterial(new BitmapTexture(new BitmapData(512, 512, true, 0xaa404070)));
			waterMaterial.alphaBlending = true;
			waterMaterial.lightPicker = lightPicker;
			waterMaterial.repeat = true;
			waterMaterial.normalMethod = waterMethod;
			waterMaterial.addMethod(new EnvMapMethod(cubeTexture));
			waterMaterial.specularMethod = fresnelMethod;
			waterMaterial.gloss = 100;
			waterMaterial.specular = 1;
		}

		override protected function initializeObjects():void
		{
			super.initializeObjects();

			scene.addChild(new SkyBox(cubeTexture));

			// create terrain
			terrain = new Elevation(terrainMaterial, Cast.bitmapData(HeightMap), 5000, 1300, 5000, 250, 250);
			scene.addChild(terrain);

			// create water
			plane = new Mesh(new PlaneGeometry(5000, 5000), waterMaterial);
			plane.geometry.scaleUV(50, 50);
			plane.y = 5;
			scene.addChild(plane);

			targetObject = camera;
			lookAtObject = terrain;
		}

		override public function prerender():void
		{
			super.prerender();

			camera.y += 0.2 * (terrain.getHeightAt(camera.x, camera.z) + 20 - camera.y);

			waterMethod.water1OffsetX += .001;
			waterMethod.water1OffsetY += .001;
			waterMethod.water2OffsetX += .0007;
			waterMethod.water2OffsetY += .0006;
		}

	}
}
