package
{
	import away3d.core.base.Object3D;
	import away3d.entities.Mesh;
	import away3d.lights.DirectionalLight;
	import away3d.materials.ColorMaterial;
	import away3d.materials.lightpickers.StaticLightPicker;
	import away3d.materials.methods.FilteredShadowMapMethod;
	import away3d.primitives.CubeGeometry;

	import caurina.transitions.Tweener;
	import caurina.transitions.properties.CurveModifiers;

	import com.blitting.display.Away3dViewport;
	import com.blitting.model.Color;
	import com.blitting.time.RandomTimer;

	import flash.events.TimerEvent;
	import flash.geom.Vector3D;
	import flash.utils.getTimer;

	[SWF(percentWidth = 100, percentHeight = 100, backgroundColor = 0x0, frameRate = 60)]
	/**
	 *
	 * Inspired by Papervision Matrix3D Tween example from wonderfl.
	 * http://wonderfl.net/c/6xey
	 *
	 * @author jsturges
	 */
	public class Away3dTweenExample extends Away3dViewport
	{

		//------------------------------
		//  model
		//------------------------------

		protected var materials:Vector.<ColorMaterial>;

		protected var meshes:Vector.<Mesh>;

		protected var light:DirectionalLight;

		protected var lightPicker:StaticLightPicker;

		protected var timer:RandomTimer;

		protected const COUNT:uint = 64;


		//------------------------------
		//  lifecycle
		//------------------------------

		/**
		 *
		 */
		public function Away3dTweenExample()
		{
			super();
		}

		/**
		 *
		 */
		override protected function initialize3d():void
		{
			super.initialize3d();

			CurveModifiers.init();

			timer = new RandomTimer(500, 1500);
			timer.addEventListener(TimerEvent.TIMER, timerHandler);
			timer.start();
		}

		/**
		 *
		 */
		override protected function initializeCamera():void
		{
			super.initializeCamera();

			camera.lens.far = 10000;
			camera.lens.near = 50;
		}

		/**
		 *
		 */
		override protected function initializeLights():void
		{
			super.initializeLights();

			light = new DirectionalLight(-300, -300, -500);
			light.color = 0xefefef;
			light.ambient = 1;
			view.scene.addChild(light);

			lightPicker = new StaticLightPicker([ light ]);
		}

		/**
		 *
		 */
		override protected function initializeMaterials():void
		{
			super.initializeMaterials();

			materials = new Vector.<ColorMaterial>();

			var f:Number = 0xff / COUNT;

			for (var i:uint = 0; i < COUNT; i++)
			{
				var material:ColorMaterial = new ColorMaterial(Color.getRGB(i * f, i * f, i * f));
				material.shadowMethod = new FilteredShadowMapMethod(light);
				material.lightPicker = lightPicker;
				materials.push(material);
			}
		}

		/**
		 *
		 */
		override protected function initializeObjects():void
		{
			super.initializeObjects();

			meshes = new Vector.<Mesh>();

			for (var i:uint = 0; i < COUNT; i++)
			{
				var mesh:Mesh = new Mesh(new CubeGeometry(20, 20, 20), materials[i]);
				mesh.castsShadows = true;

				meshes.push(mesh);
				view.scene.addChild(mesh);
			}
		}

		/**
		 *
		 */
		protected function timerHandler(event:TimerEvent):void
		{
			var sx:Number = Math.random() * 3;
			var sy:Number = Math.random() * 3;
			var sz:Number = Math.random() * 2;
			var yaw:Number = Math.random() * 1000;
			var pitch:Number = Math.random() * 1200;
			var dist:Number = Math.random() * 500 + 250;
			var lookAtZero:Boolean = Math.random() > 0.5 ? true : false;

			for (var i:uint = 0; i < meshes.length; i++)
			{
				var mesh:Mesh = meshes[i];
				var n:Number = i / meshes.length - 0.5;

				var o:Object3D = new Object3D();
				o.yaw(yaw * n);
				o.pitch(pitch * n);
				o.moveForward(dist);
				o.scaleX = sx;
				o.scaleY = sy;
				o.scaleZ = sz;

				if (lookAtZero)
					o.lookAt(Vector3D.X_AXIS);

				Tweener.addTween(mesh,
								 {
									 time: (timer.delay - 250) / 1000,
									 scaleX: o.scaleX,
									 scaleY: o.scaleY,
									 scaleZ: o.scaleZ,
									 rotationX: o.rotationX,
									 rotationY: o.rotationY,
									 rotationZ: o.rotationZ,
									 x: o.x,
									 y: o.y,
									 z: o.z,
									 _bezier:
									 {
										 x: o.x,
										 y: o.y,
										 z: o.z
									 }
								 });

				o.dispose();
				o = null;
			}
		}

		/**
		 *
		 */
		override public function render():void
		{
			super.render();

			light.direction = Vector3D.X_AXIS;
			camera.lookAt(Vector3D.X_AXIS);

			camera.x = light.x = Math.sin(getTimer() / 3000) * 1000;
			camera.z = light.z = Math.cos(getTimer() / 3000) * 1000;
		}

	}
}
