package
{
	import com.adobe.utils.AGALMiniAssembler;
	import com.blitting.display.Viewport3d;

	import flash.display3D.Context3DProgramType;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.Program3D;
	import flash.display3D.VertexBuffer3D;
	import flash.events.Event;
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	import flash.utils.ByteArray;
	import flash.utils.getTimer;

	[SWF(percentWidth = 100, percentHeight = 100, backgroundColor = 0x0, frameRate = 60)]
	/**
	 *
	 * Example shader taken from:
	 * http://www.adobe.com/devnet/flashplayer/articles/hello-triangle.html
	 *
	 */
	public class Viewport3dExample extends Viewport3d
	{

		//------------------------------
		//  model
		//------------------------------

		protected var program3d:Program3D;

		protected var vertexBuffer3d:VertexBuffer3D;

		protected var indexBuffer3d:IndexBuffer3D;

		protected var vertexShader:ByteArray;

		protected var fragmentShader:ByteArray;


		//------------------------------
		//  lifecycle
		//------------------------------

		/**
		 *
		 */
		public function Viewport3dExample(fillColor:uint=0xff000000, context3dRenderMode:String="auto")
		{
			super(fillColor, context3dRenderMode);
		}

		/**
		 *
		 */
		protected function createBuffers():void
		{
			vertexBuffer3d = context3d.createVertexBuffer(3, 6);
			indexBuffer3d = context3d.createIndexBuffer(3);
		}

		/**
		 *
		 */
		protected function createAndCompileProgram():void
		{
			program3d = context3d.createProgram();

			var assembler:AGALMiniAssembler = new AGALMiniAssembler();

			vertexShader = assembler.assemble(Context3DProgramType.VERTEX,
											  "m44 op, va0, vc0\n" + // pos to clipspace
											  "mov v0, va1" // copy color
											  );

			fragmentShader = assembler.assemble(Context3DProgramType.FRAGMENT,
												"mov oc, v0"
												);
		}

		/**
		 *
		 */
		protected function uploadProgram():void
		{
			program3d.upload(vertexShader, fragmentShader);
		}

		/**
		 *
		 */
		protected function uploadBuffers():void
		{
			var vertices:Vector.<Number> = Vector.<Number>([
														   -0.3, -0.3, 0, 1, 0, 0, // x, y, z, r, g, b
														   -0.3, 0.3, 0, 0, 1, 0,
														   0.3, 0.3, 0, 0, 0, 1 ]);

			vertexBuffer3d.uploadFromVector(vertices, 0, 3);
			indexBuffer3d.uploadFromVector(Vector.<uint>([ 0, 1, 2 ]), 0, 3);
		}

		/**
		 *
		 */
		protected function chunk():void
		{
			context3d.setVertexBufferAt(0, vertexBuffer3d, 0, Context3DVertexBufferFormat.FLOAT_3);
			context3d.setVertexBufferAt(1, vertexBuffer3d, 3, Context3DVertexBufferFormat.FLOAT_3);
		}

		/**
		 *
		 */
		protected function setActiveProgram():void
		{
			context3d.setProgram(program3d);
		}

		/**
		 *
		 */
		override protected function context3dCreateHandler(event:Event):void
		{
			super.context3dCreateHandler(event);

			createBuffers();
			createAndCompileProgram();
			uploadProgram();
			uploadBuffers();
			chunk();
			setActiveProgram();
		}

		/**
		 *
		 */
		override public function render():void
		{
			super.render();

			if (!context3d)
				return;

			var m:Matrix3D = new Matrix3D();
			m.appendRotation(getTimer() / 40, Vector3D.Z_AXIS);
			context3d.setProgramConstantsFromMatrix(Context3DProgramType.VERTEX, 0, m, true);

			context3d.drawTriangles(indexBuffer3d);
		}

	}
}
